/*!
 * @file        apm3210e_eval.c
 *
 * @brief       This file provides set of firmware functions to manage Leds, push-button and COM ports
 *              low level initialization functions for SD card (on SDIO), SPI serial flash (sFLASH) 
 *              and temperature sensor (LM75) available on APM3210E_EVAL evaluation board
 *
 * @version     V1.0.0
 *
 * @date        2020-4-14
 *
 */
  
/* Includes ------------------------------------------------------------------*/
#include "apm3210e_eval.h"
#include "apm32f10x_spi.h"
#include "apm32f10x_i2c.h"
#include "apm32f10x_sdio.h"
#include "apm32f10x_dma.h"
#include "apm32f10x_gpio.h"
#include "apm32f10x_rcm.h"
#include "apm32f10x_eint.h"
#include "misc.h"

GPIO_T* GPIO_PORT[LEDn] = {LED1_GPIO_PORT, LED2_GPIO_PORT, LED3_GPIO_PORT,
                                 LED4_GPIO_PORT};
const uint16_t GPIO_PIN[LEDn] = {LED1_PIN, LED2_PIN, LED3_PIN,
                                 LED4_PIN};
const RCM_APB2_PERIPH_T GPIO_CLK[LEDn] = {LED1_GPIO_CLK, LED2_GPIO_CLK, LED3_GPIO_CLK,
                                 LED4_GPIO_CLK};

GPIO_T* BUTTON_PORT[BUTTONn] = {WAKEUP_BUTTON_GPIO_PORT, TAMPER_BUTTON_GPIO_PORT, 
                                      KEY_BUTTON_GPIO_PORT, RIGHT_BUTTON_GPIO_PORT,
                                      LEFT_BUTTON_GPIO_PORT, UP_BUTTON_GPIO_PORT,
                                      DOWN_BUTTON_GPIO_PORT, SEL_BUTTON_GPIO_PORT}; 

const uint16_t BUTTON_PIN[BUTTONn] = {WAKEUP_BUTTON_PIN, TAMPER_BUTTON_PIN, 
                                      KEY_BUTTON_PIN, RIGHT_BUTTON_PIN,
                                      LEFT_BUTTON_PIN, UP_BUTTON_PIN,
                                      DOWN_BUTTON_PIN, SEL_BUTTON_PIN}; 

const uint32_t BUTTON_CLK[BUTTONn] = {WAKEUP_BUTTON_GPIO_CLK, TAMPER_BUTTON_GPIO_CLK,
                                      KEY_BUTTON_GPIO_CLK, RIGHT_BUTTON_GPIO_CLK,
                                      LEFT_BUTTON_GPIO_CLK, UP_BUTTON_GPIO_CLK,
                                      DOWN_BUTTON_GPIO_CLK, SEL_BUTTON_GPIO_CLK};

const uint16_t BUTTON_EXTI_LINE[BUTTONn] = {WAKEUP_BUTTON_EXTI_LINE,
                                            TAMPER_BUTTON_EXTI_LINE, 
                                            KEY_BUTTON_EXTI_LINE,
                                            RIGHT_BUTTON_EXTI_LINE,
                                            LEFT_BUTTON_EXTI_LINE,
                                            UP_BUTTON_EXTI_LINE,
                                            DOWN_BUTTON_EXTI_LINE,
                                            SEL_BUTTON_EXTI_LINE};

const uint16_t BUTTON_PORT_SOURCE[BUTTONn] = {WAKEUP_BUTTON_EXTI_PORT_SOURCE,
                                              TAMPER_BUTTON_EXTI_PORT_SOURCE, 
                                              KEY_BUTTON_EXTI_PORT_SOURCE,
                                              RIGHT_BUTTON_EXTI_PORT_SOURCE,
                                              LEFT_BUTTON_EXTI_PORT_SOURCE,
                                              UP_BUTTON_EXTI_PORT_SOURCE,
                                              DOWN_BUTTON_EXTI_PORT_SOURCE,
                                              SEL_BUTTON_EXTI_PORT_SOURCE};

const uint16_t BUTTON_PIN_SOURCE[BUTTONn] = {WAKEUP_BUTTON_EXTI_PIN_SOURCE,
                                             TAMPER_BUTTON_EXTI_PIN_SOURCE, 
                                             KEY_BUTTON_EXTI_PIN_SOURCE,
                                             RIGHT_BUTTON_EXTI_PIN_SOURCE,
                                             LEFT_BUTTON_EXTI_PIN_SOURCE,
                                             UP_BUTTON_EXTI_PIN_SOURCE,
                                             DOWN_BUTTON_EXTI_PIN_SOURCE,
                                             SEL_BUTTON_EXTI_PIN_SOURCE}; 

const uint16_t BUTTON_IRQn[BUTTONn] = {WAKEUP_BUTTON_EXTI_IRQn, TAMPER_BUTTON_EXTI_IRQn, 
                                       KEY_BUTTON_EXTI_IRQn, RIGHT_BUTTON_EXTI_IRQn,
                                       LEFT_BUTTON_EXTI_IRQn, UP_BUTTON_EXTI_IRQn,
                                       DOWN_BUTTON_EXTI_IRQn, SEL_BUTTON_EXTI_IRQn};

USART_T* COM_USART[COMn] = {EVAL_COM1, EVAL_COM2}; 

GPIO_T* COM_TX_PORT[COMn] = {EVAL_COM1_TX_GPIO_PORT, EVAL_COM2_TX_GPIO_PORT};
 
GPIO_T* COM_RX_PORT[COMn] = {EVAL_COM1_RX_GPIO_PORT, EVAL_COM2_RX_GPIO_PORT};
 
const uint32_t COM_USART_CLK[COMn] = {EVAL_COM1_CLK, EVAL_COM2_CLK};

const uint32_t COM_TX_PORT_CLK[COMn] = {EVAL_COM1_TX_GPIO_CLK, EVAL_COM2_TX_GPIO_CLK};
 
const uint32_t COM_RX_PORT_CLK[COMn] = {EVAL_COM1_RX_GPIO_CLK, EVAL_COM2_RX_GPIO_CLK};

const uint16_t COM_TX_PIN[COMn] = {EVAL_COM1_TX_PIN, EVAL_COM2_TX_PIN};

const uint16_t COM_RX_PIN[COMn] = {EVAL_COM1_RX_PIN, EVAL_COM2_RX_PIN};

/*!
 * @brief       Configures LED GPIO
 *
 * @param       Led:    Specifies the Led to be configured
 *                      This parameter can be one of following parameters
 *                      @arg LED1
 *                      @arg LED2
 *                      @arg LED3
 *                      @arg LED4
 * @retval      None
 *
 * @note       
 */
void APM_EVAL_LEDInit(Led_TypeDef Led)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;

    RCM_EnableAPB2PeriphClock((RCM_APB2_PERIPH_T)GPIO_CLK[Led]);

    GPIO_ConfigStruct.pin = GPIO_PIN[Led];
    GPIO_ConfigStruct.mode = GPIO_MODE_OUT_PP;
    GPIO_ConfigStruct.speed = GPIO_SPEED_50MHz;
    GPIO_Config(GPIO_PORT[Led], &GPIO_ConfigStruct);
}

/*!
 * @brief       Turns selected LED On
 *
 * @param       Led:    Specifies the Led to be set on
 *                      This parameter can be one of following parameters
 *                      @arg LED1
 *                      @arg LED2
 *                      @arg LED3
 *                      @arg LED4
 *
 * @retval      None
 *
 * @note       
 */
void APM_EVAL_LEDOn(Led_TypeDef Led)
{
    GPIO_PORT[Led]->BSC = GPIO_PIN[Led];     
}

/*!
 * @brief       Turns selected LED Off
 *
 * @param       Led:    Specifies the Led to be set off
 *                      This parameter can be one of following parameters
 *                      @arg LED1
 *                      @arg LED2
 *                      @arg LED3
 *                      @arg LED4
 *
 * @retval      None
 *
 * @note       
 */
void APM_EVAL_LEDOff(Led_TypeDef Led)
{
    GPIO_PORT[Led]->BC = GPIO_PIN[Led];    
}

/*!
 * @brief       Toggles the selected LED
 *
 * @param       Led:    Specifies the Led to be toggled
 *                      This parameter can be one of following parameters
 *                      @arg LED1
 *                      @arg LED2
 *                      @arg LED3
 *                      @arg LED4
 *
 * @retval      None
 *
 * @note       
 */
void APM_EVAL_LEDToggle(Led_TypeDef Led)
{
    GPIO_PORT[Led]->DOUT ^= GPIO_PIN[Led];
}

/*!
 * @brief       Configures Button GPIO and EXTI Line
 *
 * @param       Button:         Specifies the Button to be configured
 *                              This parameter can be one of following parameters
 *                              @arg BUTTON_WAKEUP: Wakeup Push Button
 *                              @arg BUTTON_TAMPER: Tamper Push Button
 *                              @arg BUTTON_KEY: Key Push Button 
 *                              @arg BUTTON_RIGHT: Joystick Right Push Button
 *                              @arg BUTTON_LEFT: Joystick Left Push Button
 *                              @arg BUTTON_UP: Joystick Up Push Button
 *                              @arg BUTTON_DOWN: Joystick Down Push Button
 *                              @arg BUTTON_SEL: Joystick Sel Push Button
 *
 * @param       Button_Mode:    Specifies Button mode
 *                              This parameter can be one of following parameters
 *                              @arg BUTTON_MODE_GPIO: Button will be used as simple IO
 *                              @arg BUTTON_MODE_EXTI: Button will be connected to EXTI line with interrupt
 * @retval      None
 *
 * @note       
 */
void APM_EVAL_PBInit(Button_TypeDef Button, ButtonMode_TypeDef Button_Mode)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;
    EINT_ConfigStruct_T EINT_ConfigStruct;

    RCM_EnableAPB2PeriphClock((RCM_APB2_PERIPH_T)(BUTTON_CLK[Button] | RCM_APB2_PERIPH_AFIO));

    GPIO_ConfigStruct.mode = GPIO_MODE_IN_FLOATING;
    GPIO_ConfigStruct.pin = BUTTON_PIN[Button];
    GPIO_Config(BUTTON_PORT[Button], &GPIO_ConfigStruct);
    
    if (Button_Mode == BUTTON_MODE_EXTI)
    {
        /* Connect Button EXTI Line to Button GPIO Pin */
        GPIO_ConfigEINTLine((GPIO_PORT_SOURCE_T)BUTTON_PORT_SOURCE[Button], (GPIO_PIN_SOURCE_T)BUTTON_PIN_SOURCE[Button]);

        /* Configure Button EXTI line */
        EINT_ConfigStruct.EINT_Line = BUTTON_EXTI_LINE[Button];
        EINT_ConfigStruct.EINT_Mode = EINT_MODE_Interrupt;

        if(Button != BUTTON_WAKEUP)
        {
            EINT_ConfigStruct.EINT_Trigger = EINT_Trigger_Falling;  
        }
        else
        {
            EINT_ConfigStruct.EINT_Trigger = EINT_Trigger_Rising;  
        }
        EINT_ConfigStruct.EINT_LineCmd = ENABLE;
        EINT_Config(&EINT_ConfigStruct);

        /* Enable and set Button EXTI Interrupt to the lowest priority */
        NVIC_EnableIRQRequest((IRQn_Type)BUTTON_IRQn[Button], 0x0F, 0x0f);
    }
}

/**
  * @brief  Returns the selected Button state.
  * @param  Button: Specifies the Button to be checked.
  *   This parameter can be one of following parameters:    
  *     @arg BUTTON_WAKEUP: Wakeup Push Button
  *     @arg BUTTON_TAMPER: Tamper Push Button  
  *     @arg BUTTON_KEY: Key Push Button 
  *     @arg BUTTON_RIGHT: Joystick Right Push Button 
  *     @arg BUTTON_LEFT: Joystick Left Push Button 
  *     @arg BUTTON_UP: Joystick Up Push Button 
  *     @arg BUTTON_DOWN: Joystick Down Push Button
  *     @arg BUTTON_SEL: Joystick Sel Push Button    
  * @retval The Button GPIO pin value.
  */
/*!
 * @brief       Returns the selected Button state
 *
 * @param       Button: Specifies the Button to be checked
 *
 * @retval      None
 *
 * @note       
 */
uint32_t APM_EVAL_PBGetState(Button_TypeDef Button)
{
  return GPIO_ReadInputBit(BUTTON_PORT[Button], BUTTON_PIN[Button]);
}

/*!
 * @brief       Configures COM port
 *
 * @param       COM                  Specifies the COM port to be configured
 *
 * @param       USART_InitStruct     pointer to a USART_InitTypeDef structure that 
 *                                    contains the configuration information for the specified USART peripheral
 * @retval      None
 *
 * @note       
 */
void APM_EVAL_COMInit(COM_TypeDef COM, USART_ConfigStruct_T* USART_InitStruct)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;

    RCM_EnableAPB2PeriphClock((RCM_APB2_PERIPH_T)(COM_TX_PORT_CLK[COM] | COM_RX_PORT_CLK[COM] | RCM_APB2_PERIPH_AFIO));

    /* Enable UART clock */
    if (COM == COM1)
    {
        RCM_EnableAPB2PeriphClock((RCM_APB2_PERIPH_T)COM_USART_CLK[COM]); 
    }
    else
    {
        RCM_EnableAPB1PeriphClock((RCM_APB1_PERIPH_T)COM_USART_CLK[COM]);
    }

    /* Configure USART Tx as alternate function push-pull */
    GPIO_ConfigStruct.mode = GPIO_MODE_OUT_PP;
    GPIO_ConfigStruct.pin = COM_TX_PIN[COM];
    GPIO_ConfigStruct.speed = GPIO_SPEED_50MHz;
    GPIO_Config(COM_TX_PORT[COM], &GPIO_ConfigStruct);

    /* Configure USART Rx as input floating */
    GPIO_ConfigStruct.mode = GPIO_MODE_IN_FLOATING;
    GPIO_ConfigStruct.pin = COM_RX_PIN[COM];
    GPIO_Config(COM_TX_PORT[COM], &GPIO_ConfigStruct);

    /* USART configuration */
    USART_Config(COM_USART[COM], USART_InitStruct);

    /* Enable USART */
    USART_Enable(COM_USART[COM]);
}

/*!
 * @brief       DeInitializes the SDIO interface
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void SD_LowLevel_DeInit(void)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;

    /*!< Disable SDIO Clock */
    SDIO_Clock_Disable();

    /*!< Set Power State to OFF */ 
    SDIO_SetPowerState(SDIO_POWER_STATE_OFF);

    /*!< DeInitializes the SDIO peripheral */
    SDIO_Reset();

    /*!< Disable the SDIO AHB Clock */
    RCM_DisableAHBPeriphClock(RCM_AHB_PERIPH_SDIO);

    /*!< Configure PC.08, PC.09, PC.10, PC.11, PC.12 pin: D0, D1, D2, D3, CLK pin */
    GPIO_ConfigStruct.pin = GPIO_PIN_8 | GPIO_PIN_9 | GPIO_PIN_10 | GPIO_PIN_11 | GPIO_PIN_12;
    GPIO_ConfigStruct.mode = GPIO_MODE_IN_FLOATING;
    GPIO_Config(GPIOC, &GPIO_ConfigStruct);

    /*!< Configure PD.02 CMD line */
    GPIO_ConfigStruct.pin = GPIO_PIN_2;
    GPIO_Config(GPIOD, &GPIO_ConfigStruct);
}

/*!
 * @brief       Initializes the SD Card and put it into StandBy State
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void SD_LowLevel_Init(void)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;

    /*!< GPIOC and GPIOD Periph clock enable */ 
    RCM_EnableAPB2PeriphClock((RCM_APB2_PERIPH_T)(RCM_APB2_PERIPH_GPIOC | RCM_APB2_PERIPH_GPIOD | SD_DETECT_GPIO_CLK));
    
    /*!< Configure PC.08, PC.09, PC.10, PC.11, PC.12 pin: D0, D1, D2, D3, CLK pin */
    GPIO_ConfigStruct.pin = GPIO_PIN_8 | GPIO_PIN_9 | GPIO_PIN_10 | GPIO_PIN_11 | GPIO_PIN_12;
    GPIO_ConfigStruct.mode = GPIO_MODE_AF_PP;
    GPIO_ConfigStruct.speed = GPIO_SPEED_50MHz;
    GPIO_Config(GPIOC, &GPIO_ConfigStruct);

    /*!< Configure PD.02 CMD line */
    GPIO_ConfigStruct.pin = GPIO_PIN_2;
    GPIO_Config(GPIOC, &GPIO_ConfigStruct);

    /*!< Configure SD_SPI_DETECT_PIN pin: SD Card detect pin */
    GPIO_ConfigStruct.pin = SD_DETECT_PIN;
    GPIO_ConfigStruct.mode = GPIO_MODE_IN_PU;
    GPIO_Config(SD_DETECT_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Enable the SDIO AHB Clock */
    RCM_EnableAHBPeriphClock(RCM_AHB_PERIPH_SDIO);

    /*!< Enable the DMA2 Clock */
    RCM_EnableAHBPeriphClock(RCM_AHB_PERIPH_DMA2);
}

/*!
 * @brief       Configures the DMA2 Channel4 for SDIO Tx request
 *
 * @param       BufferSize: pointer to the source buffer
 *
 * @param       BufferSRC:  buffer size
 *
 * @retval      None
 *
 * @note       
 */
void SD_LowLevel_DMA_TxConfig(uint32_t *BufferSRC, uint32_t BufferSize)
{
    DMA_Config_T DMA_ConfigStruct;

    DMA_ClearFlag((DMA_FLAG)(DMA2_FLAG_GIFC4 | DMA2_FLAG_EOTIFC4 | DMA2_FLAG_MOTIFC4 | DMA2_FLAG_FOTIFC4));

    /*!< DMA2 Channel4 disable */
    DMA_DISABLE(DMA2_Channel4);

    /*!< DMA2 Channel4 Config */
    DMA_ConfigStruct.PeripheralBaseAddr = (uint32_t)SDIO_FIFO_ADDRESS;
    DMA_ConfigStruct.MemoryBaseAddr = (uint32_t)BufferSRC;
    DMA_ConfigStruct.DOT = DMA_DOT_PERIPHERAL_DST;
    DMA_ConfigStruct.BufferSize = BufferSize / 4;
    DMA_ConfigStruct.PLoop = DMA_PERIPHERAL_LOOP_DISABLE;
    DMA_ConfigStruct.MLoop = DMA_MEMORY_LOOP_ENABLE;
    DMA_ConfigStruct.MemoryDataSize = DMA_MEMORY_DATA_SIZE_WOED;
    DMA_ConfigStruct.PeripheralDataSize = DMA_PERIPHERAL_DATA_SIZE_WOED;
    DMA_ConfigStruct.DMA_LoopMode = DMA_MODE_NORMAL;
    DMA_ConfigStruct.DMA_Priority = DMA_PRIORITY_HIGH;
    DMA_ConfigStruct.M2M = DMA_M2MEN_DISABLE;
    DMA_Config(DMA1_Channel4, &DMA_ConfigStruct);

    /*!< DMA2 Channel4 enable */
    DMA_ENABLE(DMA2_Channel4);
}

/*!
 * @brief       Configures the DMA2 Channel4 for SDIO Rx request
 *
 * @param       BufferDST:  pointer to the destination buffer
 *
 * @param       BufferSize: buffer size
 *
 * @retval      None
 *
 * @note       
 */
void SD_LowLevel_DMA_RxConfig(uint32_t *BufferDST, uint32_t BufferSize)
{
    DMA_Config_T DMA_ConfigStruct;

    DMA_ClearFlag((DMA_FLAG)(DMA1_FLAG_GIFC4 | DMA1_FLAG_EOTIFC4 | DMA1_FLAG_MOTIFC4 | DMA1_FLAG_FOTIFC4));

    /*!< DMA2 Channel4 disable */
    DMA_DISABLE(DMA2_Channel4);

    /*!< DMA2 Channel4 Config */
    DMA_ConfigStruct.PeripheralBaseAddr = (uint32_t)SDIO_FIFO_ADDRESS;
    DMA_ConfigStruct.MemoryBaseAddr = (uint32_t)BufferDST;
    DMA_ConfigStruct.DOT = DMA_DOT_PERIPHERAL_SRC;
    DMA_ConfigStruct.BufferSize = BufferSize / 4;
    DMA_ConfigStruct.PLoop = DMA_PERIPHERAL_LOOP_DISABLE;
    DMA_ConfigStruct.MLoop = DMA_MEMORY_LOOP_ENABLE;
    DMA_ConfigStruct.PeripheralDataSize = DMA_PERIPHERAL_DATA_SIZE_WOED;
    DMA_ConfigStruct.MemoryDataSize = DMA_MEMORY_DATA_SIZE_WOED;
    DMA_ConfigStruct.DMA_LoopMode = DMA_MODE_NORMAL;
    DMA_ConfigStruct.DMA_Priority = DMA_PRIORITY_HIGH;
    DMA_ConfigStruct.M2M = DMA_M2MEN_DISABLE;
    DMA_Config(DMA2_Channel4, &DMA_ConfigStruct);
}


/*!
 * @brief       Returns the DMA End Of Transfer Status
 *
 * @param       None
 *
 * @retval      DMA SDIO Channel Status
 *
 * @note       
 */
uint32_t SD_DMAEndOfTransferStatus(void)
{
    return (uint32_t)DMA_ReadFlagState(DMA1_FLAG_EOTIFC4);
}

/*!
 * @brief       DeInitializes the peripherals used by the SPI FLASH driver
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void sFLASH_LowLevel_DeInit(void)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;

    /*!< Disable the sFLASH_SPI  */
    SPI_Disable(sFLASH_SPI);

    /*!< DeInitializes the sFLASH_SPI */
    SPI_I2S_Reset(sFLASH_SPI);

    /*!< sFLASH_SPI Periph clock disable */
    RCM_DisableAPB2PeriphClock(sFLASH_SPI_CLK);

    /*!< Configure sFLASH_SPI pins: SCK */
    GPIO_ConfigStruct.pin = sFLASH_SPI_SCK_PIN;
    GPIO_ConfigStruct.mode = GPIO_MODE_IN_FLOATING;
    GPIO_Config(sFLASH_SPI_SCK_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure sFLASH_SPI pins: MISO */
    GPIO_ConfigStruct.pin = sFLASH_SPI_MISO_PIN;
    GPIO_Config(sFLASH_SPI_MISO_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure sFLASH_SPI pins: MOSI */
    GPIO_ConfigStruct.pin = sFLASH_SPI_MOSI_PIN;
    GPIO_Config(sFLASH_SPI_MOSI_GPIO_PORT, &GPIO_ConfigStruct);    

    /*!< Configure sFLASH_CS_PIN pin: sFLASH Card CS pin */
    GPIO_ConfigStruct.pin = sFLASH_CS_PIN;
    GPIO_Config(sFLASH_CS_GPIO_PORT, &GPIO_ConfigStruct);     
}

/*!
 * @brief       Initializes the peripherals used by the SPI FLASH driver
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void sFLASH_LowLevel_Init(void)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;

    /*!< sFLASH_SPI_CS_GPIO, sFLASH_SPI_MOSI_GPIO, sFLASH_SPI_MISO_GPIO 
       and sFLASH_SPI_SCK_GPIO Periph clock enable */
    RCM_EnableAPB2PeriphClock((RCM_APB2_PERIPH_T)(sFLASH_CS_GPIO_CLK | sFLASH_SPI_MOSI_GPIO_CLK | sFLASH_SPI_MISO_GPIO_CLK | sFLASH_SPI_SCK_GPIO_CLK));

    /*!< sFLASH_SPI Periph clock enable */
    RCM_EnableAPB2PeriphClock((RCM_APB2_PERIPH_T)sFLASH_SPI_CLK);

    /*!< Configure sFLASH_SPI pins: SCK */
    GPIO_ConfigStruct.pin = sFLASH_SPI_SCK_PIN;
    GPIO_ConfigStruct.speed = GPIO_SPEED_50MHz;
    GPIO_ConfigStruct.mode = GPIO_MODE_AF_PP;
    GPIO_Config(sFLASH_SPI_SCK_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure sFLASH_SPI pins: MOSI */
    GPIO_ConfigStruct.pin = sFLASH_SPI_MOSI_PIN;
    GPIO_Config(sFLASH_SPI_MOSI_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure sFLASH_SPI pins: MISO */
    GPIO_ConfigStruct.pin = sFLASH_SPI_MISO_PIN;
    GPIO_Config(sFLASH_SPI_MISO_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure sFLASH_CS_PIN pin: sFLASH_CS_PIN Card CS pin */
    GPIO_ConfigStruct.pin = sFLASH_CS_PIN;
    GPIO_ConfigStruct.mode = GPIO_MODE_OUT_PP;
    GPIO_Config(sFLASH_CS_GPIO_PORT, &GPIO_ConfigStruct);    
}

/*!
 * @brief       DeInitializes the LM75_I2C
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void LM75_LowLevel_DeInit(void)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;

    /*!< Disable LM75_I2C */
    I2C_Disable(LM75_I2C);
    /*!< DeInitializes the LM75_I2C */
    I2C_Reset(LM75_I2C);

    /*!< LM75_I2C Periph clock disable */
    RCM_DisableAPB1PeriphClock(LM75_I2C_CLK);

    /*!< Configure LM75_I2C pins: SCL */
    GPIO_ConfigStruct.pin = LM75_I2C_SCL_PIN;
    GPIO_ConfigStruct.mode = GPIO_MODE_IN_FLOATING;
    GPIO_Config(LM75_I2C_SCL_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure LM75_I2C pins: SDA */
    GPIO_ConfigStruct.pin = LM75_I2C_SDA_PIN;
    GPIO_Config(LM75_I2C_SCL_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure LM75_I2C pin: SMBUS ALERT */
    GPIO_ConfigStruct.pin = LM75_I2C_SMBUSALERT_PIN;
    GPIO_Config(LM75_I2C_SMBUSALERT_GPIO_PORT, &GPIO_ConfigStruct);
}

/*!
 * @brief       Initializes the LM75_I2C
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void LM75_LowLevel_Init(void)
{
    GPIO_ConfigStruct_T GPIO_ConfigStruct;

    /*!< LM75_I2C Periph clock enable */
    RCM_EnableAPB1PeriphClock(LM75_I2C_CLK);

    /*!< LM75_I2C_SCL_GPIO_CLK, LM75_I2C_SDA_GPIO_CLK 
       and LM75_I2C_SMBUSALERT_GPIO_CLK Periph clock enable */
    RCM_EnableAPB2PeriphClock((RCM_APB2_PERIPH_T)(LM75_I2C_SCL_GPIO_CLK | LM75_I2C_SDA_GPIO_CLK | LM75_I2C_SMBUSALERT_GPIO_CLK));

    /*!< Configure LM75_I2C pins: SCL */
    GPIO_ConfigStruct.pin = LM75_I2C_SCL_PIN;
    GPIO_ConfigStruct.speed = GPIO_SPEED_50MHz;
    GPIO_ConfigStruct.mode = GPIO_MODE_AF_OD;
    GPIO_Config(LM75_I2C_SCL_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure LM75_I2C pins: SDA */
    GPIO_ConfigStruct.pin = LM75_I2C_SDA_PIN;
    GPIO_Config(LM75_I2C_SDA_GPIO_PORT, &GPIO_ConfigStruct);

    /*!< Configure LM75_I2C pin: SMBUS ALERT */
    GPIO_ConfigStruct.pin = LM75_I2C_SMBUSALERT_PIN;
    GPIO_ConfigStruct.mode = GPIO_MODE_IN_PU;
    GPIO_Config(LM75_I2C_SMBUSALERT_GPIO_PORT, &GPIO_ConfigStruct); 
}

